//---------------------------------------------------------
// Vee_test.cpp
//
// Tests GC Plus and GMaster Vee control
// 
// The application ramps the Vee voltage between its limits
// in a triangle sequence (up, down, up...).
//
//
// Applied Data Systems
//
// HISTORY
// 15nov01 720110-80091
//   created based on Backlight_test (20nov00)
// - add support for GMaster and Bitsy libraries
// 11apr02 20-11061
// - change release number and display in output
// - correct bug that only supported GMaster Vee control
// - print cycle # on debug port only in debug version
//---------------------------------------------------------


#include "stdafx.h"

#define	VERSION L"20-11061"

#define DIRECT   1    // Vee PWM down -> Vee down
#define INVERSE  0    // Vee PWM up   -> Vee down

#define RAMP_UP 1
#define RAMP_DOWN 0

typedef void (*VOIDBYTEFCN)(BYTE value);
VOIDBYTEFCN SetVeePWM;

#define IOCTL_SET_BACKLIGHT_UP 		0x00
#define IOCTL_SET_BACKLIGHT_DOWN 	0x01
#define IOCTL_SET_BACKLIGHT_ON 		0x02
#define IOCTL_SET_BACKLIGHT_OFF 	0x03
#define IOCTL_SET_BACKLIGHT_LEVEL 	0x04

#define IOCTL_SET_VEE_UP 			0x10
#define IOCTL_SET_VEE_DOWN 			0x11
#define IOCTL_SET_VEE_ON 			0x12
#define IOCTL_SET_VEE_OFF 			0x13
#define IOCTL_SET_VEE_LEVEL 		0x14


//---------------------------------------------------------
//---------------------------------------------------------
void SetSA1111PWM1 (BYTE value)
{
	TCHAR SA11[] = TEXT("SAM1:");
	BOOL bRet;
	ULONG BytesRet;
	BYTE ControlVal, RetVal;
	HANDLE hPort = INVALID_HANDLE_VALUE;

	// Open SA1111 driver
	hPort = CreateFile (SA11,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
	if(hPort == INVALID_HANDLE_VALUE)
	{
		RETAILMSG(1,(L" Can't open %s. \r\n",SA11));
		return;
	}

	// Set the value
	ControlVal = value;
	bRet = DeviceIoControl(	hPort,
							IOCTL_SET_VEE_LEVEL,
							&ControlVal,
							sizeof(ControlVal),
							&RetVal,
							sizeof(RetVal),
							&BytesRet,
							NULL);
	if(bRet == FALSE)
	{
		RETAILMSG(1,(L" Error from DeviceIoControl().\r\n"));
	}
	else
	{
//		DEBUGMSG(1,(L" SA1111: ControlVal = 0x%x\r\n", ControlVal));		
	}

/*
// for test
// ramp backlight same as Vee
	bRet = DeviceIoControl(	hPort,
							IOCTL_SET_BACKLIGHT_LEVEL,
							&ControlVal,
							sizeof(ControlVal),
							&RetVal,
							sizeof(RetVal),
							&BytesRet,
							NULL);
*/

	CloseHandle(hPort);
}


//---------------------------------------------------------
//---------------------------------------------------------
void RunRamp (BYTE  Start,
			  BYTE  Stop,
			  BYTE  Step,
			  DWORD sleep_duration)
{
	int level;

	DEBUGMSG(1,(L" Ramping from %d to %d in steps of %d...\r\n",Start,Stop,Step));

	if (Start < Stop)  // ramp up
		for (level=Start;level<=Stop;level=level+Step)
		{
			DEBUGMSG(1,(L"%4d",level));
			SetVeePWM(level);
			Sleep(sleep_duration);
		}
	else               // ramp down
		for (level=Start;level>=Stop;level=level-Step)
		{
			DEBUGMSG(1,(L"%4d",level));
			SetVeePWM(level);
			Sleep(sleep_duration);
		}

	DEBUGMSG(1,(L"\r\n"));
}

//---------------------------------------------------------
// Ramps the Vee between limits of PWM
//   ramp_dir   direction to ramp; use RAMP_UP/DOWN constants
//				where RAMP_UP goes from min to max
//   polarity   use DIRECT/INVERSE constants, where
//				DIRECT means that when the PWM output voltage 
//				is 0V, Vee is at its lowest level
//---------------------------------------------------------
void VeeRamp (int ramp_dir,
			  int polarity)
{
	BYTE Min = 0;
	BYTE Max = 255;
	BYTE Step = 5;
	DWORD sleep_dur = 15;

	if (polarity==DIRECT)
		if (ramp_dir==RAMP_UP)
			RunRamp (Min,Max,Step,sleep_dur);
		else
			RunRamp (Max,Min,Step,sleep_dur);
	else										// 0V is brightest
	{
		if (ramp_dir==RAMP_UP)
			RunRamp (Max,Min,Step,sleep_dur);
		else
			RunRamp (Min,Max,Step,sleep_dur);
	}
}



//---------------------------------------------------------
// Links to the available ADSmartIO library or local
// function to perform the tests.
//---------------------------------------------------------
int SelectLibrary ()
{

  HINSTANCE gLibSMARTIO = NULL;

	// Load ADSmartIO library

	gLibSMARTIO = LoadLibrary((L"smartio.dll"));  // Graphics Master
	
  if(gLibSMARTIO == NULL)
	{	
		gLibSMARTIO = LoadLibrary((L"keybddr.dll"));  // Graphics Client Plus or Bitsy
		if(gLibSMARTIO == NULL)
		{	
			RETAILMSG(1,(L"Error: Can't find KEYBDDR.DLL or SMARTIO.DLL\r\n"));
			return 1;
		}
		else
		{	
			RETAILMSG(1,(L"Using KEYBDDR.DLL (GC Plus or Bitsy)\r\n"));
      SetVeePWM = (VOIDBYTEFCN)GetProcAddress(gLibSMARTIO, (L"SIOBackLightVEEPwm"));
		}
	}

	else  // GMaster
	{	
		RETAILMSG(1,(L"Using SMARTIO.DLL (GMaster)\r\n"));
	  SetVeePWM = (VOIDBYTEFCN)SetSA1111PWM1;
	}
	
	if(SetVeePWM ==0)
	{	
		RETAILMSG(1,(L"Error: Could not link to SIOBackLightVEEPwm.\r\n"));
		RETAILMSG(1,(L"Please use a more current build of ADS WindowsCE.\r\n"));
		return 2;
  }

  return 0;

}

//---------------------------------------------------------
//---------------------------------------------------------
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	int i, numReps=50;
	
 	RETAILMSG(1,(L"* Vee_Test (ADS #%s)\r\n",VERSION));
  RETAILMSG(1,(L"  Ramping Vee PWM between max and min %d times.\r\n",numReps));
 	RETAILMSG(1,(L"  200 ms pause when Vee level is set to 0.\r\n"));

	if (int err=SelectLibrary())
	{
		RETAILMSG(1,(L"! Program aborted.\r\n\n"));
		exit(err);
	}

	for (i=1;i<=numReps;i++)
	{
	 	DEBUGMSG(1,(L" Cycle %d of %d...\r\n",i,numReps));

		// Ramp Vee down, then back up again
		VeeRamp (RAMP_DOWN, DIRECT);

		Sleep(200);

		VeeRamp (RAMP_UP, DIRECT);

	}

 	RETAILMSG(1,(L"Vee_Test: Complete\r\n"));

	return 0;
}

